/* Copyright (c) 2013  Eric Milles <eric.milles@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "OutputMetadata.h"
#include "Uncopyable.h"


class OutputMetadataImpl
:
	private Uncopyable
{
	friend class OutputMetadata;

private:
	OutputMetadataImpl(
		time_t length,
		const std::string& title,
		const std::string& album,
		const std::string& artist,
		const buffer_t   & artworkData,
		const std::string& artworkType);
	~OutputMetadataImpl();

	time_t _length;
	std::string _title;
	std::string _album;
	std::string _artist;
	buffer_t    _artworkData;
	std::string _artworkType;
};


OutputMetadataImpl::OutputMetadataImpl(
	const time_t length,
	const std::string& title,
	const std::string& album,
	const std::string& artist,
	const buffer_t   & artworkData,
	const std::string& artworkType)
:
	_length(length),
	_title(title),
	_album(album),
	_artist(artist),
	_artworkData(artworkData),
	_artworkType(artworkType)
{
}

OutputMetadataImpl::~OutputMetadataImpl()
{
}


//------------------------------------------------------------------------------


OutputMetadata::OutputMetadata(
	const time_t length,
	const std::string& title,
	const std::string& album,
	const std::string& artist,
	const buffer_t   & artworkData,
	const std::string& artworkType)
:
	_impl(new OutputMetadataImpl(length, title, artist, album, artworkData, artworkType))
{
}


OutputMetadata::OutputMetadata(const OutputMetadata& that)
:
	_impl(new OutputMetadataImpl(
		that.length(),
		that.title(),
		that.album(),
		that.artist(),
		that.artworkData(),
		that.artworkType()))
{
}


OutputMetadata::~OutputMetadata()
{
	delete _impl;
}


OutputMetadata& OutputMetadata::operator =(const OutputMetadata& that)
{
	_impl->_length = that.length();
	_impl->_title = that.title();
	_impl->_album = that.album();
	_impl->_artist = that.artist();
	_impl->_artworkData = that.artworkData();
	_impl->_artworkType = that.artworkType();

	return *this;
}


time_t OutputMetadata::length() const
{
	return _impl->_length;
}


const std::string& OutputMetadata::title() const
{
	return _impl->_title;
}


const std::string& OutputMetadata::album() const
{
	return _impl->_album;
}


const std::string& OutputMetadata::artist() const
{
	return _impl->_artist;
}


const buffer_t   & OutputMetadata::artworkData() const
{
	return _impl->_artworkData;
}


const std::string& OutputMetadata::artworkType() const
{
	return _impl->_artworkType;
}
