/* Copyright (c) 2013  Eric Milles <eric.milles@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "PacketBuffer.h"
#include <cassert>
#include <stdexcept>


PacketBuffer::PacketBuffer(const size_t packetSize, const uint16_t head, const uint16_t tail)
:
	_packetSize(packetSize),
	_headLength(head * packetSize),
	_tailLength(tail * packetSize),
	_buffer(_headLength + _tailLength)
{
	reset();
}


PacketBuffer::~PacketBuffer()
{
}


void PacketBuffer::reset()
{
	_bufferAvailability = _headLength;
	_bufferReadIndex = _bufferWriteIndex = 0;
}


bool PacketBuffer::canWrite() const
{
	return (_bufferAvailability > 0);
}


bool PacketBuffer::canRead() const
{
	return (_bufferAvailability < _headLength);
}


byte_t* PacketBuffer::nextAvailablePacket()
{
	if (!canWrite())
	{
		throw std::logic_error("Can't write at this time");
	}

	byte_t* const ptr = &_buffer[_bufferWriteIndex];

	_bufferAvailability -= _packetSize;
	_bufferWriteIndex = (_bufferWriteIndex + _packetSize) % _buffer.size();

	return ptr;
}


byte_t* PacketBuffer::nextBufferedPacket()
{
	if (!canRead())
	{
		throw std::logic_error("Can't read at this time");
	}

	byte_t* const ptr = &_buffer[_bufferReadIndex];

	_bufferAvailability += _packetSize;
	_bufferReadIndex = (_bufferReadIndex + _packetSize) % _buffer.size();

	return ptr;
}


const byte_t* PacketBuffer::prevBufferedPacket(const uint16_t which) const
{
	if (which < 1 || which > (_tailLength / _packetSize))
	{
		throw std::logic_error("Can't find requested packet");
	}

	const size_t indexDiff = (which * _packetSize);
	const size_t prevIndex = (indexDiff <= _bufferReadIndex
		? _bufferReadIndex - indexDiff
		: _buffer.size() - (indexDiff - _bufferReadIndex));
	assert(prevIndex <= (_buffer.size() -_packetSize));
	assert(prevIndex % _packetSize == 0);

	return &_buffer[prevIndex];
}
