/* Copyright (c) 2013  Eric Milles <eric.milles@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef PacketBuffer_h
#define PacketBuffer_h


#include "Platform.h"
#include "Uncopyable.h"


class PacketBuffer
:
	private Uncopyable
{
public:
	PacketBuffer(size_t packetSize, uint16_t head, uint16_t tail = 0);
	~PacketBuffer();

	void reset();

	size_t packetSize() const;
	bool canWrite() const;
	bool canRead() const;

	      byte_t* nextAvailablePacket();
	      byte_t* nextBufferedPacket();
	const byte_t* prevBufferedPacket(uint16_t which) const;

private:
	const size_t _packetSize;
	const size_t _headLength;
	const size_t _tailLength;
	size_t _bufferAvailability;
	size_t _bufferReadIndex;
	size_t _bufferWriteIndex;
	buffer_t _buffer;
};


inline size_t PacketBuffer::packetSize() const
{
	return _packetSize;
}


#endif // PacketBuffer_h
