/* Copyright (c) 2013  Eric Milles <eric.milles@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef RTSPClient_h
#define RTSPClient_h


#include "Platform.h"
#include "Uncopyable.h"
#include "impl/Device.h"
#include <map>
#include <string>
#include <Poco/Net/StreamSocket.h>


extern const int RTSP_STATUS_CODE_OK;
extern const int RTSP_STATUS_CODE_UNAUTHORIZED;


class RTSPClient
:
	private Uncopyable
{
public:
	RTSPClient(Poco::Net::StreamSocket&, const uint32_t& remoteControlId);
	~RTSPClient();

	void setPassword(const std::string&);

	int doOptions(void* rsaKey);
	int doAnnounce(const std::string& aesKey, const std::string& aesIV);
	int doSetup(uint16_t& serverPort, uint16_t& controlPort, uint16_t& timingPort,
		unsigned int& audioLatency, AudioJackStatus&);
	int doRecord(uint16_t rtpSeqNum, uint32_t rtpTime, unsigned int& audioLatency);
	int doFlush(uint16_t rtpSeqNum, uint32_t rtpTime);
	int doTeardown();
	int doSetParameter(const std::string& key, const std::string& val);
	int doSetParameter(const std::string& contentType, const buffer_t& requestBody,
		uint32_t rtpTime);

private:
	struct Response
	{
		std::string protocol;
		std::string version;
		int statusCode;
		std::string statusText;
		std::map<const std::string,const std::string> headers;
	};

	void sendRequestReceiveResponse(const std::string& requestMethod,
		const std::string& requestHead, const std::string& requestBody, Response&);
	std::string buildRequest(const std::string& requestMethod,
		const std::string& requestHead, const std::string& requestBody) const;
	void sendRequest(const std::string& requestText);
	std::string receiveResponse();
	void parseResponse(const std::string& responseText, Response&);
	void parseAuthenticateHeader(const std::string& authenticateHeader);
	std::string buildAuthorizationHeader(const std::string& requestMethod,
		const std::string& requestURI) const;

	bool _teardownRequired;
	uint32_t _messageSequenceNumber;
	uint32_t _localSessionId;
	std::string _remoteSessionId;
	const uint32_t& _remoteControlId;

	int _authenticationCasing;
	std::string _authenticationMethod;
	std::string _authenticationRealm;
	std::string _authenticationNonce;
	std::string _authenticationPassword;

	Poco::Net::StreamSocket _rtspSocket;
};


inline void RTSPClient::setPassword(const std::string& password)
{
	_authenticationPassword = password;
}


#endif // RTSPClient_h
